@@ -110,38 +110,71 @@ class Gitlab(object):
         self._objects = objects
 
         self.broadcastmessages = objects.BroadcastMessageManager(self)
+        """See :class:`~gitlab.v4.objects.BroadcastMessageManager`"""
         self.deploykeys = objects.DeployKeyManager(self)
+        """See :class:`~gitlab.v4.objects.DeployKeyManager`"""
         self.deploytokens = objects.DeployTokenManager(self)
+        """See :class:`~gitlab.v4.objects.DeployTokenManager`"""
         self.geonodes = objects.GeoNodeManager(self)
+        """See :class:`~gitlab.v4.objects.GeoNodeManager`"""
         self.gitlabciymls = objects.GitlabciymlManager(self)
+        """See :class:`~gitlab.v4.objects.GitlabciymlManager`"""
         self.gitignores = objects.GitignoreManager(self)
+        """See :class:`~gitlab.v4.objects.GitignoreManager`"""
         self.groups = objects.GroupManager(self)
+        """See :class:`~gitlab.v4.objects.GroupManager`"""
         self.hooks = objects.HookManager(self)
+        """See :class:`~gitlab.v4.objects.HookManager`"""
         self.issues = objects.IssueManager(self)
+        """See :class:`~gitlab.v4.objects.IssueManager`"""
         self.issues_statistics = objects.IssuesStatisticsManager(self)
+        """See :class:`~gitlab.v4.objects.IssuesStatisticsManager`"""
         self.keys = objects.KeyManager(self)
+        """See :class:`~gitlab.v4.objects.KeyManager`"""
         self.ldapgroups = objects.LDAPGroupManager(self)
+        """See :class:`~gitlab.v4.objects.LDAPGroupManager`"""
         self.licenses = objects.LicenseManager(self)
+        """See :class:`~gitlab.v4.objects.LicenseManager`"""
         self.namespaces = objects.NamespaceManager(self)
+        """See :class:`~gitlab.v4.objects.NamespaceManager`"""
         self.mergerequests = objects.MergeRequestManager(self)
+        """See :class:`~gitlab.v4.objects.MergeRequestManager`"""
         self.notificationsettings = objects.NotificationSettingsManager(self)
+        """See :class:`~gitlab.v4.objects.NotificationSettingsManager`"""
         self.projects = objects.ProjectManager(self)
+        """See :class:`~gitlab.v4.objects.ProjectManager`"""
         self.runners = objects.RunnerManager(self)
+        """See :class:`~gitlab.v4.objects.RunnerManager`"""
         self.settings = objects.ApplicationSettingsManager(self)
+        """See :class:`~gitlab.v4.objects.ApplicationSettingsManager`"""
         self.appearance = objects.ApplicationAppearanceManager(self)
+        """See :class:`~gitlab.v4.objects.ApplicationAppearanceManager`"""
         self.sidekiq = objects.SidekiqManager(self)
+        """See :class:`~gitlab.v4.objects.SidekiqManager`"""
         self.snippets = objects.SnippetManager(self)
+        """See :class:`~gitlab.v4.objects.SnippetManager`"""
         self.users = objects.UserManager(self)
+        """See :class:`~gitlab.v4.objects.UserManager`"""
         self.todos = objects.TodoManager(self)
+        """See :class:`~gitlab.v4.objects.TodoManager`"""
         self.dockerfiles = objects.DockerfileManager(self)
+        """See :class:`~gitlab.v4.objects.DockerfileManager`"""
         self.events = objects.EventManager(self)
+        """See :class:`~gitlab.v4.objects.EventManager`"""
         self.audit_events = objects.AuditEventManager(self)
+        """See :class:`~gitlab.v4.objects.AuditEventManager`"""
         self.features = objects.FeatureManager(self)
+        """See :class:`~gitlab.v4.objects.FeatureManager`"""
         self.pagesdomains = objects.PagesDomainManager(self)
+        """See :class:`~gitlab.v4.objects.PagesDomainManager`"""
         self.user_activities = objects.UserActivitiesManager(self)
+        """See :class:`~gitlab.v4.objects.UserActivitiesManager`"""
         self.applications = objects.ApplicationManager(self)
+        """See :class:`~gitlab.v4.objects.ApplicationManager`"""
         self.variables = objects.VariableManager(self)
+        """See :class:`~gitlab.v4.objects.VariableManager`"""
         self.personal_access_tokens = objects.PersonalAccessTokenManager(self)
+        """See :class:`~gitlab.v4.objects.PersonalAccessTokenManager`"""
 
     def __enter__(self) -> "Gitlab":
         return self
