@@ -15,17 +15,75 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import warnings
+from types import ModuleType
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Tuple,
+    Type,
+    TYPE_CHECKING,
+    Union,
+)
+
+import requests
+
 import gitlab
-from gitlab import base
-from gitlab import cli
+from gitlab import base, cli
 from gitlab import exceptions as exc
 from gitlab import types as g_types
 from gitlab import utils
 
+__all__ = [
+    "GetMixin",
+    "GetWithoutIdMixin",
+    "RefreshMixin",
+    "ListMixin",
+    "RetrieveMixin",
+    "CreateMixin",
+    "UpdateMixin",
+    "SetMixin",
+    "DeleteMixin",
+    "CRUDMixin",
+    "NoUpdateMixin",
+    "SaveMixin",
+    "ObjectDeleteMixin",
+    "UserAgentDetailMixin",
+    "AccessRequestMixin",
+    "DownloadMixin",
+    "SubscribableMixin",
+    "TodoMixin",
+    "TimeTrackingMixin",
+    "ParticipantsMixin",
+    "BadgeRenderMixin",
+]
+
+if TYPE_CHECKING:
+    # When running mypy we use these as the base classes
+    _RestManagerBase = base.RESTManager
+    _RestObjectBase = base.RESTObject
+else:
+    _RestManagerBase = object
+    _RestObjectBase = object
+
+
+class GetMixin(_RestManagerBase):
+    _computed_path: Optional[str]
+    _from_parent_attrs: Dict[str, Any]
+    _obj_cls: Optional[Type[base.RESTObject]]
+    _optional_get_attrs: Tuple[str, ...] = ()
+    _parent: Optional[base.RESTObject]
+    _parent_attrs: Dict[str, Any]
+    _path: Optional[str]
+    gitlab: gitlab.Gitlab
 
-class GetMixin(object):
     @exc.on_http_error(exc.GitlabGetError)
-    def get(self, id, lazy=False, **kwargs):
+    def get(
+        self, id: Union[str, int], lazy: bool = False, **kwargs: Any
+    ) -> base.RESTObject:
         """Retrieve a single object.
 
         Args:
@@ -45,15 +103,32 @@ class GetMixin(object):
         if not isinstance(id, int):
             id = utils.clean_str_id(id)
         path = "%s/%s" % (self.path, id)
+        if TYPE_CHECKING:
+            assert self._obj_cls is not None
         if lazy is True:
+            if TYPE_CHECKING:
+                assert self._obj_cls._id_attr is not None
             return self._obj_cls(self, {self._obj_cls._id_attr: id})
         server_data = self.gitlab.http_get(path, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(server_data, requests.Response)
         return self._obj_cls(self, server_data)
 
 
-class GetWithoutIdMixin(object):
+class GetWithoutIdMixin(_RestManagerBase):
+    _computed_path: Optional[str]
+    _from_parent_attrs: Dict[str, Any]
+    _obj_cls: Optional[Type[base.RESTObject]]
+    _optional_get_attrs: Tuple[str, ...] = ()
+    _parent: Optional[base.RESTObject]
+    _parent_attrs: Dict[str, Any]
+    _path: Optional[str]
+    gitlab: gitlab.Gitlab
+
     @exc.on_http_error(exc.GitlabGetError)
-    def get(self, id=None, **kwargs):
+    def get(
+        self, id: Optional[Union[int, str]] = None, **kwargs: Any
+    ) -> Optional[base.RESTObject]:
         """Retrieve a single object.
 
         Args:
@@ -66,15 +141,27 @@ class GetWithoutIdMixin(object):
             GitlabAuthenticationError: If authentication is not correct
             GitlabGetError: If the server cannot perform the request
         """
+        if TYPE_CHECKING:
+            assert self.path is not None
         server_data = self.gitlab.http_get(self.path, **kwargs)
         if server_data is None:
             return None
+        if TYPE_CHECKING:
+            assert not isinstance(server_data, requests.Response)
+            assert self._obj_cls is not None
         return self._obj_cls(self, server_data)
 
 
-class RefreshMixin(object):
+class RefreshMixin(_RestObjectBase):
+    _id_attr: Optional[str]
+    _attrs: Dict[str, Any]
+    _module: ModuleType
+    _parent_attrs: Dict[str, Any]
+    _updated_attrs: Dict[str, Any]
+    manager: base.RESTManager
+
     @exc.on_http_error(exc.GitlabGetError)
-    def refresh(self, **kwargs):
+    def refresh(self, **kwargs: Any) -> None:
         """Refresh a single object from server.
 
         Args:
@@ -89,14 +176,27 @@ class RefreshMixin(object):
         if self._id_attr:
             path = "%s/%s" % (self.manager.path, self.id)
         else:
+            if TYPE_CHECKING:
+                assert self.manager.path is not None
             path = self.manager.path
         server_data = self.manager.gitlab.http_get(path, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(server_data, requests.Response)
         self._update_attrs(server_data)
 
 
-class ListMixin(object):
+class ListMixin(_RestManagerBase):
+    _computed_path: Optional[str]
+    _from_parent_attrs: Dict[str, Any]
+    _list_filters: Tuple[str, ...] = ()
+    _obj_cls: Optional[Type[base.RESTObject]]
+    _parent: Optional[base.RESTObject]
+    _parent_attrs: Dict[str, Any]
+    _path: Optional[str]
+    gitlab: gitlab.Gitlab
+
     @exc.on_http_error(exc.GitlabListError)
-    def list(self, **kwargs):
+    def list(self, **kwargs: Any) -> Union[base.RESTObjectList, List[base.RESTObject]]:
         """Retrieve a list of objects.
 
         Args:
@@ -128,9 +228,8 @@ class ListMixin(object):
             data.setdefault("order_by", self.gitlab.order_by)
 
         # We get the attributes that need some special transformation
-        types = getattr(self, "_types", {})
-        if types:
-            for attr_name, type_cls in types.items():
+        if self._types:
+            for attr_name, type_cls in self._types.items():
                 if attr_name in data.keys():
                     type_obj = type_cls(data[attr_name])
                     data[attr_name] = type_obj.get_for_api()
@@ -138,6 +237,8 @@ class ListMixin(object):
         # Allow to overwrite the path, handy for custom listings
         path = data.pop("path", self.path)
 
+        if TYPE_CHECKING:
+            assert self._obj_cls is not None
         obj = self.gitlab.http_list(path, **data)
         if isinstance(obj, list):
             return [self._obj_cls(self, item) for item in obj]
@@ -146,31 +247,39 @@ class ListMixin(object):
 
 
 class RetrieveMixin(ListMixin, GetMixin):
+    _computed_path: Optional[str]
+    _from_parent_attrs: Dict[str, Any]
+    _obj_cls: Optional[Type[base.RESTObject]]
+    _parent: Optional[base.RESTObject]
+    _parent_attrs: Dict[str, Any]
+    _path: Optional[str]
+    gitlab: gitlab.Gitlab
+
     pass
 
 
-class CreateMixin(object):
-    def _check_missing_create_attrs(self, data):
-        required, optional = self.get_create_attrs()
+class CreateMixin(_RestManagerBase):
+    _computed_path: Optional[str]
+    _from_parent_attrs: Dict[str, Any]
+    _obj_cls: Optional[Type[base.RESTObject]]
+    _parent: Optional[base.RESTObject]
+    _parent_attrs: Dict[str, Any]
+    _path: Optional[str]
+    gitlab: gitlab.Gitlab
+
+    def _check_missing_create_attrs(self, data: Dict[str, Any]) -> None:
         missing = []
-        for attr in required:
+        for attr in self._create_attrs.required:
             if attr not in data:
                 missing.append(attr)
                 continue
         if missing:
             raise AttributeError("Missing attributes: %s" % ", ".join(missing))
 
-    def get_create_attrs(self):
-        """Return the required and optional arguments.
-
-        Returns:
-            tuple: 2 items: list of required arguments and list of optional
-                   arguments for creation (in that order)
-        """
-        return getattr(self, "_create_attrs", (tuple(), tuple()))
-
     @exc.on_http_error(exc.GitlabCreateError)
-    def create(self, data=None, **kwargs):
+    def create(
+        self, data: Optional[Dict[str, Any]] = None, **kwargs: Any
+    ) -> base.RESTObject:
         """Create a new object.
 
         Args:
@@ -193,17 +302,16 @@ class CreateMixin(object):
         files = {}
 
         # We get the attributes that need some special transformation
-        types = getattr(self, "_types", {})
-        if types:
+        if self._types:
             # Duplicate data to avoid messing with what the user sent us
             data = data.copy()
-            for attr_name, type_cls in types.items():
+            for attr_name, type_cls in self._types.items():
                 if attr_name in data.keys():
                     type_obj = type_cls(data[attr_name])
 
                     # if the type if FileAttribute we need to pass the data as
                     # file
-                    if issubclass(type_cls, g_types.FileAttribute):
+                    if isinstance(type_obj, g_types.FileAttribute):
                         k = type_obj.get_file_name(attr_name)
                         files[attr_name] = (k, data.pop(attr_name))
                     else:
@@ -212,14 +320,30 @@ class CreateMixin(object):
         # Handle specific URL for creation
         path = kwargs.pop("path", self.path)
         server_data = self.gitlab.http_post(path, post_data=data, files=files, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(server_data, requests.Response)
+            assert self._obj_cls is not None
         return self._obj_cls(self, server_data)
 
 
-class UpdateMixin(object):
-    def _check_missing_update_attrs(self, data):
-        required, optional = self.get_update_attrs()
-        # Remove the id field from the required list as it was previously moved to the http path.
-        required = tuple(filter(lambda k: k != self._obj_cls._id_attr, required))
+class UpdateMixin(_RestManagerBase):
+    _computed_path: Optional[str]
+    _from_parent_attrs: Dict[str, Any]
+    _obj_cls: Optional[Type[base.RESTObject]]
+    _parent: Optional[base.RESTObject]
+    _parent_attrs: Dict[str, Any]
+    _path: Optional[str]
+    _update_uses_post: bool = False
+    gitlab: gitlab.Gitlab
+
+    def _check_missing_update_attrs(self, data: Dict[str, Any]) -> None:
+        if TYPE_CHECKING:
+            assert self._obj_cls is not None
+        # Remove the id field from the required list as it was previously moved
+        # to the http path.
+        required = tuple(
+            [k for k in self._update_attrs.required if k != self._obj_cls._id_attr]
+        )
         missing = []
         for attr in required:
             if attr not in data:
@@ -228,29 +352,27 @@ class UpdateMixin(object):
         if missing:
             raise AttributeError("Missing attributes: %s" % ", ".join(missing))
 
-    def get_update_attrs(self):
-        """Return the required and optional arguments.
-
-        Returns:
-            tuple: 2 items: list of required arguments and list of optional
-                   arguments for update (in that order)
-        """
-        return getattr(self, "_update_attrs", (tuple(), tuple()))
-
-    def _get_update_method(self):
+    def _get_update_method(
+        self,
+    ) -> Callable[..., Union[Dict[str, Any], requests.Response]]:
         """Return the HTTP method to use.
 
         Returns:
             object: http_put (default) or http_post
         """
-        if getattr(self, "_update_uses_post", False):
+        if self._update_uses_post:
             http_method = self.gitlab.http_post
         else:
             http_method = self.gitlab.http_put
         return http_method
 
     @exc.on_http_error(exc.GitlabUpdateError)
-    def update(self, id=None, new_data=None, **kwargs):
+    def update(
+        self,
+        id: Optional[Union[str, int]] = None,
+        new_data: Optional[Dict[str, Any]] = None,
+        **kwargs: Any
+    ) -> Dict[str, Any]:
         """Update an object on the server.
 
         Args:
@@ -276,29 +398,39 @@ class UpdateMixin(object):
         files = {}
 
         # We get the attributes that need some special transformation
-        types = getattr(self, "_types", {})
-        if types:
+        if self._types:
             # Duplicate data to avoid messing with what the user sent us
             new_data = new_data.copy()
-            for attr_name, type_cls in types.items():
+            for attr_name, type_cls in self._types.items():
                 if attr_name in new_data.keys():
                     type_obj = type_cls(new_data[attr_name])
 
                     # if the type if FileAttribute we need to pass the data as
                     # file
-                    if issubclass(type_cls, g_types.FileAttribute):
+                    if isinstance(type_obj, g_types.FileAttribute):
                         k = type_obj.get_file_name(attr_name)
                         files[attr_name] = (k, new_data.pop(attr_name))
                     else:
                         new_data[attr_name] = type_obj.get_for_api()
 
         http_method = self._get_update_method()
-        return http_method(path, post_data=new_data, files=files, **kwargs)
+        result = http_method(path, post_data=new_data, files=files, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(result, requests.Response)
+        return result
 
 
-class SetMixin(object):
+class SetMixin(_RestManagerBase):
+    _computed_path: Optional[str]
+    _from_parent_attrs: Dict[str, Any]
+    _obj_cls: Optional[Type[base.RESTObject]]
+    _parent: Optional[base.RESTObject]
+    _parent_attrs: Dict[str, Any]
+    _path: Optional[str]
+    gitlab: gitlab.Gitlab
+
     @exc.on_http_error(exc.GitlabSetError)
-    def set(self, key, value, **kwargs):
+    def set(self, key: str, value: str, **kwargs: Any) -> base.RESTObject:
         """Create or update the object.
 
         Args:
@@ -316,12 +448,23 @@ class SetMixin(object):
         path = "%s/%s" % (self.path, utils.clean_str_id(key))
         data = {"value": value}
         server_data = self.gitlab.http_put(path, post_data=data, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(server_data, requests.Response)
+            assert self._obj_cls is not None
         return self._obj_cls(self, server_data)
 
 
-class DeleteMixin(object):
+class DeleteMixin(_RestManagerBase):
+    _computed_path: Optional[str]
+    _from_parent_attrs: Dict[str, Any]
+    _obj_cls: Optional[Type[base.RESTObject]]
+    _parent: Optional[base.RESTObject]
+    _parent_attrs: Dict[str, Any]
+    _path: Optional[str]
+    gitlab: gitlab.Gitlab
+
     @exc.on_http_error(exc.GitlabDeleteError)
-    def delete(self, id, **kwargs):
+    def delete(self, id: Union[str, int], **kwargs: Any) -> None:
         """Delete an object on the server.
 
         Args:
@@ -342,20 +485,42 @@ class DeleteMixin(object):
 
 
 class CRUDMixin(GetMixin, ListMixin, CreateMixin, UpdateMixin, DeleteMixin):
+    _computed_path: Optional[str]
+    _from_parent_attrs: Dict[str, Any]
+    _obj_cls: Optional[Type[base.RESTObject]]
+    _parent: Optional[base.RESTObject]
+    _parent_attrs: Dict[str, Any]
+    _path: Optional[str]
+    gitlab: gitlab.Gitlab
+
     pass
 
 
 class NoUpdateMixin(GetMixin, ListMixin, CreateMixin, DeleteMixin):
+    _computed_path: Optional[str]
+    _from_parent_attrs: Dict[str, Any]
+    _obj_cls: Optional[Type[base.RESTObject]]
+    _parent: Optional[base.RESTObject]
+    _parent_attrs: Dict[str, Any]
+    _path: Optional[str]
+    gitlab: gitlab.Gitlab
+
     pass
 
 
-class SaveMixin(object):
+class SaveMixin(_RestObjectBase):
     """Mixin for RESTObject's that can be updated."""
 
-    def _get_updated_data(self):
+    _id_attr: Optional[str]
+    _attrs: Dict[str, Any]
+    _module: ModuleType
+    _parent_attrs: Dict[str, Any]
+    _updated_attrs: Dict[str, Any]
+    manager: base.RESTManager
+
+    def _get_updated_data(self) -> Dict[str, Any]:
         updated_data = {}
-        required, optional = self.manager.get_update_attrs()
-        for attr in required:
+        for attr in self.manager._update_attrs.required:
             # Get everything required, no matter if it's been updated
             updated_data[attr] = getattr(self, attr)
         # Add the updated attributes
@@ -363,7 +528,7 @@ class SaveMixin(object):
 
         return updated_data
 
-    def save(self, **kwargs):
+    def save(self, **kwargs: Any) -> None:
         """Save the changes made to the object to the server.
 
         The object is updated to match what the server returns.
@@ -382,15 +547,24 @@ class SaveMixin(object):
 
         # call the manager
         obj_id = self.get_id()
+        if TYPE_CHECKING:
+            assert isinstance(self.manager, UpdateMixin)
         server_data = self.manager.update(obj_id, updated_data, **kwargs)
         if server_data is not None:
             self._update_attrs(server_data)
 
 
-class ObjectDeleteMixin(object):
+class ObjectDeleteMixin(_RestObjectBase):
     """Mixin for RESTObject's that can be deleted."""
 
-    def delete(self, **kwargs):
+    _id_attr: Optional[str]
+    _attrs: Dict[str, Any]
+    _module: ModuleType
+    _parent_attrs: Dict[str, Any]
+    _updated_attrs: Dict[str, Any]
+    manager: base.RESTManager
+
+    def delete(self, **kwargs: Any) -> None:
         """Delete the object from the server.
 
         Args:
@@ -400,13 +574,22 @@ class ObjectDeleteMixin(object):
             GitlabAuthenticationError: If authentication is not correct
             GitlabDeleteError: If the server cannot perform the request
         """
-        self.manager.delete(self.get_id())
+        if TYPE_CHECKING:
+            assert isinstance(self.manager, DeleteMixin)
+        self.manager.delete(self.get_id(), **kwargs)
 
 
-class UserAgentDetailMixin(object):
+class UserAgentDetailMixin(_RestObjectBase):
+    _id_attr: Optional[str]
+    _attrs: Dict[str, Any]
+    _module: ModuleType
+    _parent_attrs: Dict[str, Any]
+    _updated_attrs: Dict[str, Any]
+    manager: base.RESTManager
+
     @cli.register_custom_action(("Snippet", "ProjectSnippet", "ProjectIssue"))
     @exc.on_http_error(exc.GitlabGetError)
-    def user_agent_detail(self, **kwargs):
+    def user_agent_detail(self, **kwargs: Any) -> Dict[str, Any]:
         """Get the user agent detail.
 
         Args:
@@ -417,15 +600,27 @@ class UserAgentDetailMixin(object):
             GitlabGetError: If the server cannot perform the request
         """
         path = "%s/%s/user_agent_detail" % (self.manager.path, self.get_id())
-        return self.manager.gitlab.http_get(path, **kwargs)
+        result = self.manager.gitlab.http_get(path, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(result, requests.Response)
+        return result
 
 
-class AccessRequestMixin(object):
+class AccessRequestMixin(_RestObjectBase):
+    _id_attr: Optional[str]
+    _attrs: Dict[str, Any]
+    _module: ModuleType
+    _parent_attrs: Dict[str, Any]
+    _updated_attrs: Dict[str, Any]
+    manager: base.RESTManager
+
     @cli.register_custom_action(
         ("ProjectAccessRequest", "GroupAccessRequest"), tuple(), ("access_level",)
     )
     @exc.on_http_error(exc.GitlabUpdateError)
-    def approve(self, access_level=gitlab.DEVELOPER_ACCESS, **kwargs):
+    def approve(
+        self, access_level: int = gitlab.DEVELOPER_ACCESS, **kwargs: Any
+    ) -> None:
         """Approve an access request.
 
         Args:
@@ -440,13 +635,28 @@ class AccessRequestMixin(object):
         path = "%s/%s/approve" % (self.manager.path, self.id)
         data = {"access_level": access_level}
         server_data = self.manager.gitlab.http_put(path, post_data=data, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(server_data, requests.Response)
         self._update_attrs(server_data)
 
 
-class DownloadMixin(object):
+class DownloadMixin(_RestObjectBase):
+    _id_attr: Optional[str]
+    _attrs: Dict[str, Any]
+    _module: ModuleType
+    _parent_attrs: Dict[str, Any]
+    _updated_attrs: Dict[str, Any]
+    manager: base.RESTManager
+
     @cli.register_custom_action(("GroupExport", "ProjectExport"))
     @exc.on_http_error(exc.GitlabGetError)
-    def download(self, streamed=False, action=None, chunk_size=1024, **kwargs):
+    def download(
+        self,
+        streamed: bool = False,
+        action: Optional[Callable] = None,
+        chunk_size: int = 1024,
+        **kwargs: Any
+    ) -> Optional[bytes]:
         """Download the archive of a resource export.
 
         Args:
@@ -469,15 +679,24 @@ class DownloadMixin(object):
         result = self.manager.gitlab.http_get(
             path, streamed=streamed, raw=True, **kwargs
         )
+        if TYPE_CHECKING:
+            assert isinstance(result, requests.Response)
         return utils.response_content(result, streamed, action, chunk_size)
 
 
-class SubscribableMixin(object):
+class SubscribableMixin(_RestObjectBase):
+    _id_attr: Optional[str]
+    _attrs: Dict[str, Any]
+    _module: ModuleType
+    _parent_attrs: Dict[str, Any]
+    _updated_attrs: Dict[str, Any]
+    manager: base.RESTManager
+
     @cli.register_custom_action(
         ("ProjectIssue", "ProjectMergeRequest", "ProjectLabel", "GroupLabel")
     )
     @exc.on_http_error(exc.GitlabSubscribeError)
-    def subscribe(self, **kwargs):
+    def subscribe(self, **kwargs: Any) -> None:
         """Subscribe to the object notifications.
 
         Args:
@@ -489,13 +708,15 @@ class SubscribableMixin(object):
         """
         path = "%s/%s/subscribe" % (self.manager.path, self.get_id())
         server_data = self.manager.gitlab.http_post(path, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(server_data, requests.Response)
         self._update_attrs(server_data)
 
     @cli.register_custom_action(
         ("ProjectIssue", "ProjectMergeRequest", "ProjectLabel", "GroupLabel")
     )
     @exc.on_http_error(exc.GitlabUnsubscribeError)
-    def unsubscribe(self, **kwargs):
+    def unsubscribe(self, **kwargs: Any) -> None:
         """Unsubscribe from the object notifications.
 
         Args:
@@ -507,13 +728,22 @@ class SubscribableMixin(object):
         """
         path = "%s/%s/unsubscribe" % (self.manager.path, self.get_id())
         server_data = self.manager.gitlab.http_post(path, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(server_data, requests.Response)
         self._update_attrs(server_data)
 
 
-class TodoMixin(object):
+class TodoMixin(_RestObjectBase):
+    _id_attr: Optional[str]
+    _attrs: Dict[str, Any]
+    _module: ModuleType
+    _parent_attrs: Dict[str, Any]
+    _updated_attrs: Dict[str, Any]
+    manager: base.RESTManager
+
     @cli.register_custom_action(("ProjectIssue", "ProjectMergeRequest"))
     @exc.on_http_error(exc.GitlabTodoError)
-    def todo(self, **kwargs):
+    def todo(self, **kwargs: Any) -> None:
         """Create a todo associated to the object.
 
         Args:
@@ -527,10 +757,17 @@ class TodoMixin(object):
         self.manager.gitlab.http_post(path, **kwargs)
 
 
-class TimeTrackingMixin(object):
+class TimeTrackingMixin(_RestObjectBase):
+    _id_attr: Optional[str]
+    _attrs: Dict[str, Any]
+    _module: ModuleType
+    _parent_attrs: Dict[str, Any]
+    _updated_attrs: Dict[str, Any]
+    manager: base.RESTManager
+
     @cli.register_custom_action(("ProjectIssue", "ProjectMergeRequest"))
     @exc.on_http_error(exc.GitlabTimeTrackingError)
-    def time_stats(self, **kwargs):
+    def time_stats(self, **kwargs: Any) -> Dict[str, Any]:
         """Get time stats for the object.
 
         Args:
@@ -546,11 +783,14 @@ class TimeTrackingMixin(object):
             return self.attributes["time_stats"]
 
         path = "%s/%s/time_stats" % (self.manager.path, self.get_id())
-        return self.manager.gitlab.http_get(path, **kwargs)
+        result = self.manager.gitlab.http_get(path, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(result, requests.Response)
+        return result
 
     @cli.register_custom_action(("ProjectIssue", "ProjectMergeRequest"), ("duration",))
     @exc.on_http_error(exc.GitlabTimeTrackingError)
-    def time_estimate(self, duration, **kwargs):
+    def time_estimate(self, duration: str, **kwargs: Any) -> Dict[str, Any]:
         """Set an estimated time of work for the object.
 
         Args:
@@ -563,11 +803,14 @@ class TimeTrackingMixin(object):
         """
         path = "%s/%s/time_estimate" % (self.manager.path, self.get_id())
         data = {"duration": duration}
-        return self.manager.gitlab.http_post(path, post_data=data, **kwargs)
+        result = self.manager.gitlab.http_post(path, post_data=data, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(result, requests.Response)
+        return result
 
     @cli.register_custom_action(("ProjectIssue", "ProjectMergeRequest"))
     @exc.on_http_error(exc.GitlabTimeTrackingError)
-    def reset_time_estimate(self, **kwargs):
+    def reset_time_estimate(self, **kwargs: Any) -> Dict[str, Any]:
         """Resets estimated time for the object to 0 seconds.
 
         Args:
@@ -578,11 +821,14 @@ class TimeTrackingMixin(object):
             GitlabTimeTrackingError: If the time tracking update cannot be done
         """
         path = "%s/%s/reset_time_estimate" % (self.manager.path, self.get_id())
-        return self.manager.gitlab.http_post(path, **kwargs)
+        result = self.manager.gitlab.http_post(path, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(result, requests.Response)
+        return result
 
     @cli.register_custom_action(("ProjectIssue", "ProjectMergeRequest"), ("duration",))
     @exc.on_http_error(exc.GitlabTimeTrackingError)
-    def add_spent_time(self, duration, **kwargs):
+    def add_spent_time(self, duration: str, **kwargs: Any) -> Dict[str, Any]:
         """Add time spent working on the object.
 
         Args:
@@ -595,11 +841,14 @@ class TimeTrackingMixin(object):
         """
         path = "%s/%s/add_spent_time" % (self.manager.path, self.get_id())
         data = {"duration": duration}
-        return self.manager.gitlab.http_post(path, post_data=data, **kwargs)
+        result = self.manager.gitlab.http_post(path, post_data=data, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(result, requests.Response)
+        return result
 
     @cli.register_custom_action(("ProjectIssue", "ProjectMergeRequest"))
     @exc.on_http_error(exc.GitlabTimeTrackingError)
-    def reset_spent_time(self, **kwargs):
+    def reset_spent_time(self, **kwargs: Any) -> Dict[str, Any]:
         """Resets the time spent working on the object.
 
         Args:
@@ -610,13 +859,23 @@ class TimeTrackingMixin(object):
             GitlabTimeTrackingError: If the time tracking update cannot be done
         """
         path = "%s/%s/reset_spent_time" % (self.manager.path, self.get_id())
-        return self.manager.gitlab.http_post(path, **kwargs)
+        result = self.manager.gitlab.http_post(path, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(result, requests.Response)
+        return result
 
 
-class ParticipantsMixin(object):
+class ParticipantsMixin(_RestObjectBase):
+    _id_attr: Optional[str]
+    _attrs: Dict[str, Any]
+    _module: ModuleType
+    _parent_attrs: Dict[str, Any]
+    _updated_attrs: Dict[str, Any]
+    manager: base.RESTManager
+
     @cli.register_custom_action(("ProjectMergeRequest", "ProjectIssue"))
     @exc.on_http_error(exc.GitlabListError)
-    def participants(self, **kwargs):
+    def participants(self, **kwargs: Any) -> Dict[str, Any]:
         """List the participants.
 
         Args:
@@ -636,15 +895,18 @@ class ParticipantsMixin(object):
         """
 
         path = "%s/%s/participants" % (self.manager.path, self.get_id())
-        return self.manager.gitlab.http_get(path, **kwargs)
+        result = self.manager.gitlab.http_get(path, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(result, requests.Response)
+        return result
 
 
-class BadgeRenderMixin(object):
+class BadgeRenderMixin(_RestManagerBase):
     @cli.register_custom_action(
         ("GroupBadgeManager", "ProjectBadgeManager"), ("link_url", "image_url")
     )
     @exc.on_http_error(exc.GitlabRenderError)
-    def render(self, link_url, image_url, **kwargs):
+    def render(self, link_url: str, image_url: str, **kwargs: Any) -> Dict[str, Any]:
         """Preview link_url and image_url after interpolation.
 
         Args:
@@ -661,4 +923,54 @@ class BadgeRenderMixin(object):
         """
         path = "%s/render" % self.path
         data = {"link_url": link_url, "image_url": image_url}
-        return self.gitlab.http_get(path, data, **kwargs)
+        result = self.gitlab.http_get(path, data, **kwargs)
+        if TYPE_CHECKING:
+            assert not isinstance(result, requests.Response)
+        return result
+
+
+class MemberAllMixin(_RestManagerBase):
+    """This mixin is deprecated."""
+
+    _computed_path: Optional[str]
+    _from_parent_attrs: Dict[str, Any]
+    _obj_cls: Optional[Type[base.RESTObject]]
+    _parent: Optional[base.RESTObject]
+    _parent_attrs: Dict[str, Any]
+    _path: Optional[str]
+    gitlab: gitlab.Gitlab
+
+    @cli.register_custom_action(("GroupMemberManager", "ProjectMemberManager"))
+    @exc.on_http_error(exc.GitlabListError)
+    def all(self, **kwargs: Any) -> List[base.RESTObject]:
+        """List all the members, included inherited ones.
+
+        This Method is deprecated.
+
+        Args:
+            all (bool): If True, return all the items, without pagination
+            per_page (int): Number of items to retrieve per request
+            page (int): ID of the page to return (starts with page 1)
+            as_list (bool): If set to False and no pagination option is
+                defined, return a generator instead of a list
+            **kwargs: Extra options to send to the server (e.g. sudo)
+
+        Raises:
+            GitlabAuthenticationError: If authentication is not correct
+            GitlabListError: If the list could not be retrieved
+
+        Returns:
+            RESTObjectList: The list of members
+        """
+
+        warnings.warn(
+            "The all() method for this object is deprecated "
+            "and will be removed in a future version. Use .members_all.list(all=True), instead.",
+            DeprecationWarning,
+        )
+        path = "%s/all" % self.path
+
+        if TYPE_CHECKING:
+            assert self._obj_cls is not None
+        obj = self.gitlab.http_list(path, **kwargs)
+        return [self._obj_cls(self, item) for item in obj]
