@@ -1,9 +1,10 @@
-from gitlab import cli, types
+from gitlab import cli
 from gitlab import exceptions as exc
+from gitlab import types
 from gitlab.base import RequiredOptional, RESTManager, RESTObject
 from gitlab.mixins import (
-    CRUDMixin,
     CreateMixin,
+    CRUDMixin,
     DeleteMixin,
     ListMixin,
     ObjectDeleteMixin,
@@ -15,6 +16,7 @@ from gitlab.mixins import (
     TodoMixin,
     UserAgentDetailMixin,
 )
+
 from .award_emojis import ProjectIssueAwardEmojiManager  # noqa: F401
 from .discussions import ProjectIssueDiscussionManager  # noqa: F401
 from .events import (  # noqa: F401
@@ -24,7 +26,6 @@ from .events import (  # noqa: F401
 )
 from .notes import ProjectIssueNoteManager  # noqa: F401
 
-
 __all__ = [
     "Issue",
     "IssueManager",
@@ -62,7 +63,7 @@ class IssueManager(RetrieveMixin, RESTManager):
         "updated_after",
         "updated_before",
     )
-    _types = {"labels": types.ListAttribute}
+    _types = {"iids": types.ListAttribute, "labels": types.ListAttribute}
 
 
 class GroupIssue(RESTObject):
@@ -89,7 +90,7 @@ class GroupIssueManager(ListMixin, RESTManager):
         "updated_after",
         "updated_before",
     )
-    _types = {"labels": types.ListAttribute}
+    _types = {"iids": types.ListAttribute, "labels": types.ListAttribute}
 
 
 class ProjectIssue(
@@ -220,7 +221,7 @@ class ProjectIssueManager(CRUDMixin, RESTManager):
             "discussion_locked",
         ),
     )
-    _types = {"labels": types.ListAttribute}
+    _types = {"iids": types.ListAttribute, "labels": types.ListAttribute}
 
 
 class ProjectIssueLink(ObjectDeleteMixin, RESTObject):
