@@ -23,6 +23,24 @@ def test_create_project_release(project, project_file):
     assert release.description == release_description
 
 
+def test_create_project_release_no_name(project, project_file):
+    unnamed_release_tag_name = "v2.3.4"
+
+    project.refresh()  # Gets us the current default branch
+    release = project.releases.create(
+        {
+            "tag_name": unnamed_release_tag_name,
+            "description": release_description,
+            "ref": project.default_branch,
+        }
+    )
+
+    assert len(project.releases.list()) >= 1
+    assert project.releases.get(unnamed_release_tag_name)
+    assert release.tag_name == unnamed_release_tag_name
+    assert release.description == release_description
+
+
 def test_update_save_project_release(project, release):
     updated_description = f"{release.description} updated"
     release.description = updated_description
